use crate::{AppState, auth::User, err, partials::page, succ};
use axum::{
    Form, Router,
    extract::State,
    response::{IntoResponse, Redirect},
    routing::{get, post},
};
use maud::{Markup, html};
use serde::Deserialize;
use sqlx::SqlitePool;
use std::sync::Arc;
use tower_sessions::Session;

#[derive(Deserialize)]
struct CreateForm {
    name: String,
}

async fn create(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    Form(form): Form<CreateForm>,
) -> impl IntoResponse {
    let id = match User::create(&db, &form.name).await {
        Ok(id) => {
            succ!(session, "User '{}' erfolgreich erstellt!", form.name);
            id
        }
        Err(e) => {
            err!(
                session,
                "User '{}' konnte _NICHT_ erstellt werden: {e:?}",
                e
            );

            return Redirect::to("/admin/user");
        }
    };

    Redirect::to(&format!("/admin/user/{id}"))
}

async fn delete(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> impl IntoResponse {
    let Some(user) = User::find_by_id(&db, id).await else {
        err!(
            session,
            "User mit ID {id} konnte nicht gelöscht werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/user");
    };

    match user.delete(&db).await {
        Ok(()) => succ!(session, "User '{}' erfolgreich gelöscht!", user.name),
        Err(e) => err!(
            session,
            "User '{}' kann nicht gelöscht werden, da er/sie bereits verwendet wird. ({e})",
            user.name
        ),
    }

    Redirect::to("/admin/user")
}

async fn view(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> Result<Markup, impl IntoResponse> {
    let Some(user) = User::find_by_id(&db, id).await else {
        err!(
            session,
            "User mit ID {id} konnte nicht geöffnet werden, da sie nicht existiert"
        );

        return Err(Redirect::to("/admin/user"));
    };

    // maybe switch to maud-display impl of team
    let content = html! {
        h1 {
            a href="/admin/user" { "↩️" }
            "User " (user.name)
        }
        article {
            details {
                summary { "Name bearbeiten ✏️" }
                form action=(format!("/admin/user/{}/name", user.id)) method="post" {
                    input type="text" name="name" value=(user.name) required;
                    input type="submit" value="Speichern";
                }

            }
        }
        table {
            tbody {
                tr {
                    th scope="row" { "Name" };
                    td {
                        (user.name)
                    }
                }
                tr {
                    th scope="row" { "Passwort" };
                    td {
                        @if let Some(code) = user.require_new_password_code {
                            a href=(format!("/user/{}/set-pw/{}", user.id, code)){
                                "Login-Link"
                            }
                        } @else {
                            a href=(format!("/admin/user/{}/new-pw", user.id))
                                onclick="return confirm('Bist du sicher, dass du einen neuen Passwort-Link generieren willst? Mit dem alten Passwort kann man sich dann nicht mehr einloggen.');" {
                                    "Passwort vergessen: neuen Loginlink generieren"
                            }
                        }
                    }
                }
            }
        }
    };
    Ok(page(content, session, true).await)
}

#[derive(Deserialize)]
struct UpdateNameForm {
    name: String,
}
async fn update_name(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
    Form(form): Form<UpdateNameForm>,
) -> impl IntoResponse {
    let Some(user) = User::find_by_id(&db, id).await else {
        err!(
            session,
            "User mit ID {id} konnte nicht bearbeitet werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/user");
    };

    user.update_name(&db, &form.name).await;

    succ!(
        session,
        "User '{}' heißt ab sofort '{}'.",
        user.name,
        form.name
    );

    Redirect::to(&format!("/admin/user/{id}"))
}

async fn new_pw(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> impl IntoResponse {
    let Some(user) = User::find_by_id(&db, id).await else {
        err!(
            session,
            "User mit ID {id} konnte nicht bearbeitet werden, da sie nicht existiert"
        );

        return Redirect::to("/admin/user");
    };

    user.new_pw(&db).await;

    succ!(
        session,
        "Neuer Loginlink für User '{}' wurde generiert.",
        user.name
    );

    Redirect::to(&format!("/admin/user/{id}"))
}

async fn index(State(db): State<Arc<SqlitePool>>, session: Session) -> Markup {
    let users = User::all(&db).await;

    let content = html! {
        h1 {
            a href="/admin" { "↩️" }
            "User"
        }
        article {
            em { "Admins " }
            "sind Menschen, die "
            a href="/admin/station" { "Stationen" }
            ", "
            a href="/admin/route" { "Routen" }
            " und "
            a href="/admin/team" { "Teams" }
            " bearbeiten können. Zusätzlich sehen sie das "
            a href="/admin/highscore" { "Ergebnis" }
            "."
        }
        article {
            h2 { "Neuer User" }
                form action="/admin/user" method="post" {
                        fieldset role="group" {
                            input type="text" name="name" placeholder="Name" required;
                            input type="submit" value="Neuer User";
                        }
                }
        }
        ul {
                @for user in &users {
                    li {
                         a href=(format!("/admin/user/{}", user.id)){
                            (user.name)
                         }
                         a href=(format!("/admin/user/{}/delete", user.id))
                           onclick="return confirm('Bist du sicher, dass der User gelöscht werden soll? Das kann _NICHT_ mehr rückgängig gemacht werden.');" {
                             "🗑️"
                         }
                    }
                }
        }
    };
    page(content, session, false).await
}

pub(super) fn routes() -> Router<AppState> {
    Router::new()
        .route("/", get(index))
        .route("/", post(create))
        .route("/{id}", get(view))
        .route("/{id}/delete", get(delete))
        .route("/{id}/new-pw", get(new_pw))
        .route("/{id}/name", post(update_name))
}
